@@ -301,6 +301,8 @@ class WandbCallback(keras.callbacks.Callback):
     ):
         if wandb.run is None:
             raise wandb.Error("You must call wandb.init() before WandbCallback()")
+        with wandb.wandb_lib.telemetry.context(run=wandb.run) as tel:
+            tel.feature.keras = True
 
         self.validation_data = None
         # This is kept around for legacy reasons
