@@ -5,6 +5,7 @@ import logging
 import numbers
 import os
 import shutil
+import sys
 
 import six
 from six.moves.collections_abc import Sequence as SixSequence
@@ -214,12 +215,12 @@ class Histogram(WBValue):
 
     Examples:
         Generate histogram from a sequence
-        ```
+        ```python
         wandb.Histogram([1,2,3])
         ```
 
         Efficiently initialize from np.histogram.
-        ```
+        ```python
         hist = np.histogram(data)
         wandb.Histogram(np_histogram=hist)
         ```
@@ -278,6 +279,12 @@ class Histogram(WBValue):
     def to_json(self, run: Union["LocalRun", "LocalArtifact"] = None) -> dict:
         return {"_type": "histogram", "values": self.histogram, "bins": self.bins}
 
+    def __sizeof__(self) -> int:
+        """This returns an estimated size in bytes, currently the factor of 1.7
+        is used to account for the JSON encoding.  We use this in tb_watcher.TBHistory
+        """
+        return int((sys.getsizeof(self.histogram) + sys.getsizeof(self.bins)) * 1.7)
+
 
 class Media(WBValue):
     """A WBValue that we store as a file outside JSON and show in a media panel
@@ -403,6 +410,10 @@ class Media(WBValue):
         Returns:
             dict: JSON representation
         """
+        # NOTE: uses of Audio in this class are a temporary hack -- when Ref support moves up
+        # into Media itself we should get rid of them
+        from wandb.data_types import Audio
+
         json_obj = {}
         run_class, artifact_class = _safe_sdk_import()
         if isinstance(run, run_class):
@@ -471,6 +482,10 @@ class Media(WBValue):
                         # Add this image as a reference
                         path = self.artifact_source.artifact.get_path(name)
                         artifact.add_reference(path.ref_url(), name=name)
+                    elif isinstance(self, Audio) and Audio.path_is_reference(
+                        self._path
+                    ):
+                        artifact.add_reference(self._path, name=name)
                     else:
                         entry = artifact.add_file(
                             self._path, name=name, is_tmp=self._is_tmp
@@ -532,7 +547,7 @@ class Object3D(BatchableMedia):
                 a file or an io object and a file_type which must be one of `'obj', 'gltf', 'babylon', 'stl'`.
 
     The shape of the numpy array must be one of either:
-    ```
+    ```python
     [[x y z],       ...] nx3
     [x y z c],     ...] nx4 where c is a category with supported range [1, 14]
     [x y z r g b], ...] nx4 where is rgb is color
@@ -540,7 +555,7 @@ class Object3D(BatchableMedia):
     """
 
     SUPPORTED_TYPES: ClassVar[Set[str]] = set(
-        ["obj", "gltf", "babylon", "stl", "pts.json"]
+        ["obj", "gltf", "glb", "babylon", "stl", "pts.json"]
     )
     artifact_type: ClassVar[str] = "object3D-file"
 
@@ -1999,9 +2014,11 @@ def val_to_json(
     typename = util.get_full_typename(val)
 
     if util.is_pandas_data_frame(val):
-        assert run
-        assert namespace == "summary", "We don't yet support DataFrames in History."
-        return _data_frame_to_json(val, run, key, namespace)
+        raise ValueError(
+            "We do not support DataFrames in the Summary or History. Try run.log({{'{}': wandb.Table(dataframe=df)}})".format(
+                key
+            )
+        )
     elif util.is_matplotlib_typename(typename) or util.is_plotly_typename(typename):
         val = Plotly.make_plot_media(val)
     elif isinstance(val, SixSequence) and all(isinstance(v, WBValue) for v in val):
