@@ -11,7 +11,7 @@ from wandb import util
 from wandb.data_types import WBValue
 
 if wandb.TYPE_CHECKING:  # type: ignore
-    from typing import Optional, Union
+    from typing import List, Optional, Union, Dict
 
 
 def md5_string(string):
@@ -93,7 +93,7 @@ class ArtifactManifest(object):
 
 
 class ArtifactEntry(object):
-    def parent_artifact(self):
+    def parent_artifact(self) -> "Artifact":
         """
         Get the artifact to which this artifact entry belongs.
 
@@ -147,6 +147,7 @@ class ArtifactEntry(object):
 
 
 class Artifact(object):
+    @property
     def id(self) -> Optional[str]:
         """
         Returns:
@@ -154,15 +155,33 @@ class Artifact(object):
         """
         raise NotImplementedError
 
-    def metadata(self) -> dict:
+    @property
+    def version(self) -> str:
         """
         Returns:
-            (dict): Structured data associated with the artifact,
-                for example class distribution of a dataset. This will eventually be queryable
-                and plottable in the UI. There is a hard limit of 100 total keys.
+            (int): The version of this artifact. For example, if this
+                is the first version of an artifact, its `version` will
+                be 'v0'.
         """
         raise NotImplementedError
 
+    @property
+    def name(self) -> str:
+        """
+        Returns:
+            (str): The artifact's name
+        """
+        raise NotImplementedError
+
+    @property
+    def type(self) -> str:
+        """
+        Returns:
+            (str): The artifact's type
+        """
+        raise NotImplementedError
+
+    @property
     def entity(self) -> str:
         """
         Returns:
@@ -170,6 +189,7 @@ class Artifact(object):
         """
         raise NotImplementedError
 
+    @property
     def project(self) -> str:
         """
         Returns:
@@ -177,6 +197,7 @@ class Artifact(object):
         """
         raise NotImplementedError
 
+    @property
     def manifest(self) -> ArtifactManifest:
         """
         Returns:
@@ -186,6 +207,7 @@ class Artifact(object):
         """
         raise NotImplementedError
 
+    @property
     def digest(self) -> str:
         """
         Returns:
@@ -195,6 +217,7 @@ class Artifact(object):
         """
         raise NotImplementedError
 
+    @property
     def state(self) -> str:
         """
         Returns:
@@ -203,6 +226,7 @@ class Artifact(object):
         """
         raise NotImplementedError
 
+    @property
     def size(self) -> int:
         """
         Returns:
@@ -211,6 +235,77 @@ class Artifact(object):
         """
         raise NotImplementedError
 
+    @property
+    def description(self) -> Optional[str]:
+        """
+        Returns:
+            (str): Free text that offers a description of the artifact. The
+                description is markdown rendered in the UI, so this is a good place
+                to put links, etc.
+        """
+        raise NotImplementedError
+
+    @description.setter
+    def description(self, desc: Optional[str]) -> None:
+        """
+        Arguments:
+            desc: Free text that offers a description of the artifact. The
+                description is markdown rendered in the UI, so this is a good place
+                to put links, etc.
+        """
+        raise NotImplementedError
+
+    @property
+    def metadata(self) -> dict:
+        """
+        Returns:
+            (dict): Structured data associated with the artifact,
+                for example class distribution of a dataset. This will eventually be queryable
+                and plottable in the UI. There is a hard limit of 100 total keys.
+        """
+        raise NotImplementedError
+
+    @metadata.setter
+    def metadata(self, metadata: dict) -> None:
+        """
+        Arguments:
+            metadata: (dict) Structured data associated with the artifact,
+                for example class distribution of a dataset. This will eventually be queryable
+                and plottable in the UI. There is a hard limit of 100 total keys.
+        """
+        raise NotImplementedError
+
+    @property
+    def aliases(self) -> List[str]:
+        """
+        Returns:
+            (list): A list of the aliases associated with this artifact. The list is
+                mutable and calling `save()` will persist all alias changes.
+        """
+        raise NotImplementedError
+
+    @aliases.setter
+    def aliases(self, aliases: List[str]) -> None:
+        """
+        Arguments:
+            aliases: (list) The list of aliases associated with this artifact.
+        """
+        raise NotImplementedError
+
+    def used_by(self) -> List["wandb.apis.public.Run"]:
+        """
+        Returns:
+            (list): A list of the runs that have used this artifact.
+        """
+        raise NotImplementedError
+
+    def logged_by(self) -> "wandb.apis.public.Run":
+        """
+        Returns:
+            (Run): The run that first logged this artifact.
+        """
+        raise NotImplementedError
+
     def new_file(self, name: str, mode: str = "w"):
         """
         Open a new temporary file that will be automatically added to the artifact.
@@ -269,7 +364,7 @@ class Artifact(object):
         """
         raise NotImplementedError
 
-    def add_dir(self, local_path: str, name: Optional[str] = None):
+    def add_dir(self, local_path: str, name: Optional[str] = None) -> None:
         """
         Adds a local directory to the artifact.
 
@@ -469,12 +564,76 @@ class Artifact(object):
         match the artifact.
 
         Arguments:
-            root: (str, optional) The directory in which to download this artifact's files
+            root: (str, optional) The directory in which to download this artifact's files.
             recursive: (bool, optional) If true, then all dependent artifacts are eagerly
                 downloaded. Otherwise, the dependent artifacts are downloaded as needed.
 
         Returns:
-            The path to the downloaded contents.
+            (str): The path to the downloaded contents.
+        """
+        raise NotImplementedError
+
+    def checkout(self, root: Optional[str] = None) -> str:
+        """
+        Replaces the specified root directory with the contents of the artifact.
+
+        WARNING: This will DELETE all files in `root` that are not included in the
+        artifact.
+
+        Arguments:
+            root: (str, optional) The directory to replace with this artifact's files.
+
+        Returns:
+           (str): The path to the checked out contents.
+        """
+        raise NotImplementedError
+
+    def verify(self, root: Optional[str] = None):
+        """
+        Verify that the actual contents of an artifact at a specified directory
+        `root` match the expected contents of the artifact according to its
+        manifest.
+
+        All files in the directory are checksummed and the checksums are then
+        cross-referenced against the artifact's manifest.
+
+        NOTE: References are not verified.
+
+        Arguments:
+            root: (str, optional) The directory to verify. If None
+                artifact will be downloaded to './artifacts/<self.name>/'
+
+        Raises:
+            (ValueError): If the verification fails.
+        """
+        raise NotImplementedError
+
+    def save(self) -> None:
+        """
+        Persists any changes made to the artifact.
+
+        Returns:
+            None
+        """
+        raise NotImplementedError
+
+    def delete(self) -> None:
+        """
+        Deletes this artifact, cleaning up all files associated with it.
+
+        NOTE: Deletion is permanent and CANNOT be undone.
+
+        Returns:
+            None
+        """
+        raise NotImplementedError
+
+    def wait(self) -> "Artifact":
+        """
+        Waits for this artifact to finish logging, if needed.
+
+        Returns:
+            Artifact
         """
         raise NotImplementedError
 
@@ -583,11 +742,36 @@ class ArtifactsCache(object):
     def store_artifact(self, artifact):
         self._artifacts_by_id[artifact.id] = artifact
 
+    def cleanup(self, target_size: int) -> int:
+        bytes_reclaimed: int = 0
+        paths: Dict[os.PathLike, os.stat_result] = {}
+        total_size: int = 0
+        for root, _, files in os.walk(self._cache_dir):
+            for file in files:
+                path = os.path.join(root, file)
+                stat_res = os.stat(path)
+                paths[path] = stat_res
+                total_size += stat_res.st_size
+
+        sorted_paths = sorted(paths.items(), key=lambda x: x[1].st_atime)
+        for path, stat in sorted_paths:
+            if total_size < target_size:
+                return bytes_reclaimed
+
+            try:
+                os.remove(path)
+            except OSError:
+                pass
+
+            total_size -= stat.st_size
+            bytes_reclaimed += stat.st_size
+        return bytes_reclaimed
+
 
 _artifacts_cache = None
 
 
-def get_artifacts_cache():
+def get_artifacts_cache() -> ArtifactsCache:
     global _artifacts_cache
     if _artifacts_cache is None:
         cache_dir = os.path.join(env.get_cache_dir(), "artifacts")
