@@ -22,6 +22,7 @@ from wandb.util import (
     json_dumps_safer,
     json_dumps_safer_history,
     json_friendly,
+    json_friendly_val,
     maybe_compress_summary,
     WandBJSONEncoderOld,
 )
@@ -246,7 +247,7 @@ class BackendSender(object):
         if artifact.description:
             proto_artifact.description = artifact.description
         if artifact.metadata:
-            proto_artifact.metadata = json.dumps(artifact.metadata)
+            proto_artifact.metadata = json.dumps(json_friendly_val(artifact.metadata))  # type: ignore
         self._make_artifact_manifest(artifact.manifest, obj=proto_artifact.manifest)
         return proto_artifact
 
@@ -423,6 +424,7 @@ class BackendSender(object):
         sampled_history: pb.SampledHistoryRequest = None,
         run_start: pb.RunStartRequest = None,
         check_version: pb.CheckVersionRequest = None,
+        log_artifact: pb.LogArtifactRequest = None,
         defer: pb.DeferRequest = None,
     ) -> pb.Record:
         request = pb.Request()
@@ -444,6 +446,8 @@ class BackendSender(object):
             request.run_start.CopyFrom(run_start)
         elif check_version:
             request.check_version.CopyFrom(check_version)
+        elif log_artifact:
+            request.log_artifact.CopyFrom(log_artifact)
         elif defer:
             request.defer.CopyFrom(defer)
         else:
@@ -520,10 +524,12 @@ class BackendSender(object):
     def _communicate(
         self, rec: pb.Record, timeout: Optional[int] = 5, local: bool = None
     ) -> Optional[pb.Result]:
+        return self._communicate_async(rec, local=local).get(timeout=timeout)
+
+    def _communicate_async(self, rec: pb.Record, local: bool = None) -> _Future:
         assert self._router
         future = self._router.send_and_receive(rec, local=local)
-        f = future.get(timeout)
-        return f
+        return future
 
     def communicate_login(
         self, api_key: str = None, timeout: Optional[int] = 15
@@ -649,6 +655,31 @@ class BackendSender(object):
         rec = self._make_record(files=files)
         self._publish(rec)
 
+    def communicate_artifact(
+        self,
+        run: "Run",
+        artifact: Artifact,
+        aliases: Iterable[str],
+        is_user_created: bool = False,
+        use_after_commit: bool = False,
+        finalize: bool = True,
+    ) -> _Future:
+        proto_run = self._make_run(run)
+        proto_artifact = self._make_artifact(artifact)
+        proto_artifact.run_id = proto_run.run_id
+        proto_artifact.project = proto_run.project
+        proto_artifact.entity = proto_run.entity
+        proto_artifact.user_created = is_user_created
+        proto_artifact.use_after_commit = use_after_commit
+        proto_artifact.finalize = finalize
+        for alias in aliases:
+            proto_artifact.aliases.append(alias)
+
+        log_artifact = pb.LogArtifactRequest()
+        log_artifact.artifact.CopyFrom(proto_artifact)
+        rec = self._make_request(log_artifact=log_artifact)
+        return self._communicate_async(rec)
+
     def publish_artifact(
         self,
         run: "Run",
