@@ -6,6 +6,7 @@ tensor b watcher.
 import logging
 import os
 import socket
+import sys
 import threading
 import time
 
@@ -13,6 +14,7 @@ import six
 from six.moves import queue
 import wandb
 from wandb import util
+from wandb.viz import custom_chart_panel_config, CustomChart
 
 from . import run as internal_run
 
@@ -105,12 +107,14 @@ class TBWatcher(object):
         settings: "SettingsStatic",
         run_proto: "RunRecord",
         interface: "BackendSender",
+        force: bool = False,
     ) -> None:
         self._logdirs = {}
         self._consumer = None
         self._settings = settings
         self._interface = interface
         self._run_proto = run_proto
+        self._force = force
         # TODO(jhr): do we need locking in this queue?
         self._watcher_queue = queue.PriorityQueue()
         wandb.tensorboard.reset_state()
@@ -140,10 +144,12 @@ class TBWatcher(object):
                 namespace = None
         else:
             namespace = logdir.replace(filename, "").replace(rootdir, "").strip("/")
+
         return namespace
 
     def add(self, logdir: str, save: bool, root_dir: str) -> None:
         logdir = util.to_forward_slash_path(logdir)
+        root_dir = util.to_forward_slash_path(root_dir)
         if logdir in self._logdirs:
             return
         namespace = self._calculate_namespace(logdir, root_dir)
@@ -155,7 +161,9 @@ class TBWatcher(object):
             )
             self._consumer.start()
 
-        tbdir_watcher = TBDirWatcher(self, logdir, save, namespace, self._watcher_queue)
+        tbdir_watcher = TBDirWatcher(
+            self, logdir, save, namespace, self._watcher_queue, self._force
+        )
         self._logdirs[logdir] = tbdir_watcher
         tbdir_watcher.start()
 
@@ -176,6 +184,7 @@ class TBDirWatcher(object):
         save: bool,
         namespace: "Optional[str]",
         queue: "PriorityQueue",
+        force: bool = False,
     ) -> None:
         self.directory_watcher = util.get_module(
             "tensorboard.backend.event_processing.directory_watcher",
@@ -200,6 +209,7 @@ class TBDirWatcher(object):
         self._namespace = namespace
         self._logdir = logdir
         self._hostname = socket.gethostname()
+        self._force = force
 
     def start(self) -> None:
         self._thread.start()
@@ -208,6 +218,8 @@ class TBDirWatcher(object):
         """Checks if a path has been modified since launch and contains tfevents"""
         if not path:
             raise ValueError("Path must be a nonempty string")
+        if self._force:
+            return True
         path = self.tf_compat.tf.compat.as_str_any(path)
         return is_tfevents_file_created_by(
             path, self._hostname, self._tbwatcher._settings._start_time
@@ -246,22 +258,26 @@ class TBDirWatcher(object):
 
         return EventFileLoader
 
+    def _process_events(self, shutdown_call: bool = False) -> None:
+        try:
+            for event in self._generator.Load():
+                self.process_event(event)
+        except (
+            self.directory_watcher.DirectoryDeletedError,
+            StopIteration,
+            RuntimeError,
+            OSError,
+        ) as e:
+            # When listing s3 the directory may not yet exist, or could be empty
+            logger.debug("Encountered tensorboard directory watcher error: %s", e)
+            if not self._shutdown.is_set() and not shutdown_call:
+                time.sleep(ERROR_DELAY)
+
     def _thread_body(self) -> None:
         """Check for new events every second"""
         shutdown_time: "Optional[float]" = None
         while True:
-            try:
-                for event in self._generator.Load():
-                    self.process_event(event)
-            except (
-                self.directory_watcher.DirectoryDeletedError,
-                StopIteration,
-                RuntimeError,
-            ) as e:
-                # When listing s3 the directory may not yet exist, or could be empty
-                logger.debug("Encountered tensorboard directory watcher error: %s", e)
-                if not self._shutdown.is_set():
-                    time.sleep(ERROR_DELAY)
+            self._process_events()
             if self._shutdown.is_set():
                 now = time.time()
                 if not shutdown_time:
@@ -282,6 +298,7 @@ class TBDirWatcher(object):
             self._queue.put(Event(event, self._namespace))
 
     def shutdown(self) -> None:
+        self._process_events(shutdown_call=True)
         self._shutdown.set()
 
     def finish(self) -> None:
@@ -379,6 +396,22 @@ class TBEventConsumer(object):
         )
 
     def _save_row(self, row: "HistoryDict") -> None:
+        chart_keys = []
+        for key, item in row.items():
+            if isinstance(item, CustomChart):
+                panel_config = custom_chart_panel_config(item, key, key + "_table")
+                config = {"panel_type": "Vega2", "panel_config": panel_config}
+                chart_keys.append(key)
+                self._tbwatcher._interface.publish_config(
+                    val=config, key=("_wandb", "visualize", key)
+                )
+                row[key] = item.table
+
+        for chart_key in chart_keys:
+            table = row[chart_key]
+            row.pop(chart_key)
+            row[chart_key + "_table"] = table
+
         self._tbwatcher._interface.publish_history(
             row, run=self._internal_run, publish_step=False
         )
@@ -390,23 +423,54 @@ class TBHistory(object):
 
     def __init__(self) -> None:
         self._step = 0
+        self._step_size = 0
         self._data = dict()
         self._added = []
 
     def _flush(self) -> None:
         if not self._data:
             return
+        # A single tensorboard step may have too much data
+        # we just drop the largest keys in the step if it does.
+        # TODO: we could flush the data across multiple steps
+        if self._step_size > util.MAX_LINE_SIZE:
+            metrics = [(k, sys.getsizeof(v)) for k, v in self._data.items()]
+            metrics.sort(key=lambda t: t[1], reverse=True)
+            bad = 0
+            dropped_keys = []
+            for k, v in metrics:
+                # TODO: (cvp) Added a buffer of 100KiB, this feels rather brittle.
+                if self._step_size - bad < util.MAX_LINE_SIZE - 100000:
+                    break
+                else:
+                    bad += v
+                    dropped_keys.append(k)
+                    del self._data[k]
+            wandb.termwarn(
+                "Step {} exceeds max data limit, dropping {} of the largest keys:".format(
+                    self._step, len(dropped_keys)
+                )
+            )
+            print("\t" + ("\n\t".join(dropped_keys)))
         self._data["_step"] = self._step
         self._added.append(self._data)
         self._step += 1
+        self._step_size = 0
 
     def add(self, d: "HistoryDict") -> None:
         self._flush()
         self._data = dict()
-        self._data.update(d)
+        self._data.update(self._track_history_dict(d))
+
+    def _track_history_dict(self, d: "HistoryDict") -> "HistoryDict":
+        e = {}
+        for k in d.keys():
+            e[k] = d[k]
+            self._step_size += sys.getsizeof(e[k])
+        return e
 
     def _row_update(self, d: "HistoryDict") -> None:
-        self._data.update(d)
+        self._data.update(self._track_history_dict(d))
 
     def _get_and_reset(self) -> "List[HistoryDict]":
         added = self._added[:]
