@@ -19,10 +19,11 @@ from wandb.data_types import WBValue
 from wandb.errors.error import CommError
 from wandb.errors.term import termlog, termwarn
 
-from .interface.artifacts import (
+from .interface.artifacts import (  # noqa: F401 pylint: disable=unused-import
     Artifact as ArtifactInterface,
     ArtifactEntry,
     ArtifactManifest,
+    ArtifactsCache,
     b64_string_to_hex,
     get_artifacts_cache,
     md5_file_b64,
@@ -32,7 +33,7 @@ from .interface.artifacts import (
 )
 
 if wandb.TYPE_CHECKING:  # type: ignore
-    from typing import Optional, Union
+    from typing import List, Optional, Union
 
 # This makes the first sleep 1s, and then doubles it up to total times,
 # which makes for ~18 hours.
@@ -90,6 +91,7 @@ class Artifact(ArtifactInterface):
     _added_local_paths: dict
     _distributed_id: Optional[str]
     _metadata: dict
+    _logged_artifact: Optional[ArtifactInterface]
 
     def __init__(
         self,
@@ -132,64 +134,137 @@ class Artifact(ArtifactInterface):
         self._description = description
         self._metadata = metadata or {}
         self._distributed_id = None
+        self._logged_artifact = None
 
     @property
     def id(self) -> Optional[str]:
+        if self._logged_artifact:
+            return self._logged_artifact.id
+
         # The artifact hasn't been saved so an ID doesn't exist yet.
         return None
 
+    @property
+    def version(self) -> str:
+        if self._logged_artifact:
+            return self._logged_artifact.version
+
+        raise ValueError(
+            "Cannot call version on an artifact before it has been logged or in offline mode"
+        )
+
     @property
     def entity(self) -> str:
-        # TODO: querying for default entity a good idea here?
+        if self._logged_artifact:
+            return self._logged_artifact.entity
+
         return self._api.settings("entity") or self._api.viewer().get("entity")
 
     @property
     def project(self) -> str:
+        if self._logged_artifact:
+            return self._logged_artifact.project
+
         return self._api.settings("project")
 
     @property
     def manifest(self) -> ArtifactManifest:
+        if self._logged_artifact:
+            return self._logged_artifact.manifest
+
         self.finalize()
         return self._manifest
 
     @property
     def digest(self) -> str:
+        if self._logged_artifact:
+            return self._logged_artifact.digest
+
         self.finalize()
         # Digest will be none if the artifact hasn't been saved yet.
         return self._digest
 
-    @property
-    def description(self) -> Optional[str]:
-        return self._description
-
-    @description.setter
-    def description(self, desc: Optional[str]) -> None:
-        self._description = desc
-
-    @property
-    def metadata(self) -> dict:
-        return self._metadata
-
-    @metadata.setter
-    def metadata(self, metadata: dict) -> None:
-        self._metadata = metadata
-
     @property
     def type(self) -> str:
+        if self._logged_artifact:
+            return self._logged_artifact.type
+
         return self._type
 
     @property
     def name(self) -> str:
+        if self._logged_artifact:
+            return self._logged_artifact.name
+
         return self._name
 
     @property
     def state(self) -> str:
+        if self._logged_artifact:
+            return self._logged_artifact.state
+
         return "PENDING"
 
     @property
     def size(self) -> int:
+        if self._logged_artifact:
+            return self._logged_artifact.size
+
         return sum([entry.size for entry in self._manifest.entries])
 
+    @property
+    def description(self) -> Optional[str]:
+        if self._logged_artifact:
+            return self._logged_artifact.description
+
+        return self._description
+
+    @description.setter
+    def description(self, desc: Optional[str]) -> None:
+        if self._logged_artifact:
+            self._logged_artifact.description = desc
+            return
+
+        self._description = desc
+
+    @property
+    def metadata(self) -> dict:
+        if self._logged_artifact:
+            return self._logged_artifact.metadata
+
+        return self._metadata
+
+    @metadata.setter
+    def metadata(self, metadata: dict) -> None:
+        if self._logged_artifact:
+            self._metadata = metadata
+            return
+
+        self._metadata = metadata
+
+    @property
+    def aliases(self) -> List[str]:
+        if self._logged_artifact:
+            return self._logged_artifact.aliases
+
+        raise ValueError(
+            "Cannot call aliases on an artifact before it has been logged or in offline mode"
+        )
+
+    @aliases.setter
+    def aliases(self, aliases: List[str]) -> None:
+        """
+        Arguments:
+            aliases: (list) The list of aliases associated with this artifact.
+        """
+        if self._logged_artifact:
+            self._logged_artifact.aliases = aliases
+            return
+
+        raise ValueError(
+            "Cannot set aliases on an artifact before it has been logged or in offline mode"
+        )
+
     @property
     def distributed_id(self) -> Optional[str]:
         return self._distributed_id
@@ -198,9 +273,21 @@ class Artifact(ArtifactInterface):
     def distributed_id(self, distributed_id: Optional[str]) -> None:
         self._distributed_id = distributed_id
 
-    def _ensure_can_add(self):
-        if self._final:
-            raise ValueError("Can't add to finalized artifact.")
+    def used_by(self) -> List["wandb.apis.public.Run"]:
+        if self._logged_artifact:
+            return self._logged_artifact.used_by()
+
+        raise ValueError(
+            "Cannot call used_by on an artifact before it has been logged or in offline mode"
+        )
+
+    def logged_by(self) -> "wandb.apis.public.Run":
+        if self._logged_artifact:
+            return self._logged_artifact.logged_by()
+
+        raise ValueError(
+            "Cannot call logged_by on an artifact before it has been logged or in offline mode"
+        )
 
     @contextlib.contextmanager
     def new_file(self, name: str, mode: str = "w"):
@@ -238,7 +325,7 @@ class Artifact(ArtifactInterface):
 
         return self._add_local_file(name, local_path, digest=digest)
 
-    def add_dir(self, local_path: str, name: Optional[str] = None):
+    def add_dir(self, local_path: str, name: Optional[str] = None) -> None:
         self._ensure_can_add()
         if not os.path.isdir(local_path):
             raise ValueError("Path is not a directory: %s" % local_path)
@@ -340,13 +427,68 @@ class Artifact(ArtifactInterface):
         return entry
 
     def get_path(self, name: str):
-        raise ValueError("Cannot load paths from an artifact before it has been saved")
+        if self._logged_artifact:
+            return self._logged_artifact.get_path(name)
+
+        raise ValueError(
+            "Cannot load paths from an artifact before it has been logged or in offline mode"
+        )
 
     def get(self, name: str):
-        raise ValueError("Cannot call get on an artifact before it has been saved")
+        if self._logged_artifact:
+            return self._logged_artifact.get(name)
+
+        raise ValueError(
+            "Cannot call get on an artifact before it has been logged or in offline mode"
+        )
 
     def download(self, root: str = None, recursive: bool = False):
-        raise ValueError("Cannot call download on an artifact before it has been saved")
+        if self._logged_artifact:
+            return self._logged_artifact.download(root=root, recursive=recursive)
+
+        raise ValueError(
+            "Cannot call download on an artifact before it has been logged or in offline mode"
+        )
+
+    def checkout(self, root: Optional[str] = None) -> str:
+        if self._logged_artifact:
+            return self._logged_artifact.checkout(root=root)
+
+        raise ValueError(
+            "Cannot call checkout on an artifact before it has been logged or in offline mode"
+        )
+
+    def verify(self, root: Optional[str] = None):
+        if self._logged_artifact:
+            return self._logged_artifact.verify(root=root)
+
+        raise ValueError(
+            "Cannot call verify on an artifact before it has been logged or in offline mode"
+        )
+
+    def save(self) -> None:
+        if self._logged_artifact:
+            return self._logged_artifact.save()
+
+        raise ValueError(
+            "Cannot call save on an artifact before it has been logged or in offline mode"
+        )
+
+    def delete(self) -> None:
+        if self._logged_artifact:
+            return self._logged_artifact.delete()
+
+        raise ValueError(
+            "Cannot call delete on an artifact before it has been logged or in offline mode"
+        )
+
+    def wait(self) -> ArtifactInterface:
+        if self._logged_artifact:
+            return self._logged_artifact.wait()
+
+        raise ValueError(
+            "Cannot call wait on an artifact before it has been logged or in offline mode"
+        )
 
     def get_added_local_path_name(self, local_path: str):
         """
@@ -389,6 +531,10 @@ class Artifact(ArtifactInterface):
         self._final = True
         self._digest = self._manifest.digest()
 
+    def _ensure_can_add(self):
+        if self._final:
+            raise ValueError("Can't add to finalized artifact.")
+
     def _add_local_file(self, name, path, digest=None):
         digest = digest or md5_file_b64(path)
         size = os.path.getsize(path)
@@ -898,6 +1044,9 @@ class S3Handler(StorageHandler):
         return self._versioning_enabled
 
     def load_path(self, artifact, manifest_entry, local=False):
+        if not local:
+            return manifest_entry.ref
+
         path, hit = self._cache.check_etag_obj_path(
             manifest_entry.digest, manifest_entry.size
         )
@@ -943,9 +1092,6 @@ class S3Handler(StorageHandler):
             obj = self._s3.ObjectVersion(bucket, key, version).Object()
             extra_args["VersionId"] = version
 
-        if not local:
-            return manifest_entry.ref
-
         obj.download_file(path, ExtraArgs=extra_args)
         return path
 
@@ -1086,6 +1232,9 @@ class GCSHandler(StorageHandler):
         return bucket, key
 
     def load_path(self, artifact, manifest_entry, local=False):
+        if not local:
+            return manifest_entry.ref
+
         path, hit = self._cache.check_md5_obj_path(
             manifest_entry.digest, manifest_entry.size
         )
@@ -1117,9 +1266,6 @@ class GCSHandler(StorageHandler):
                     % (manifest_entry.ref, manifest_entry.digest, md5)
                 )
 
-        if not local:
-            return manifest_entry.ref
-
         obj.download_to_filename(path)
         return path
 
@@ -1302,7 +1448,10 @@ class WBArtifactHandler(StorageHandler):
         dep_artifact = PublicArtifact.from_id(
             util.hex_to_b64_id(artifact_id), self.client
         )
-        link_target_path = dep_artifact.get_path(artifact_file_path).download()
+        if local:
+            link_target_path = dep_artifact.get_path(artifact_file_path).download()
+        else:
+            link_target_path = dep_artifact.get_path(artifact_file_path).ref()
 
         return link_target_path
 
