@@ -155,21 +155,28 @@ class _WandbInit(object):
         # apply updated global state after login was handled
         settings._apply_settings(wandb.setup()._settings)
 
-        settings._apply_init(kwargs)
+        # get status of code saving before applying user settings
+        save_code_pre_user_settings = settings["save_code"]
 
+        settings._apply_init(kwargs)
         if not settings._offline and not settings._noop:
             user_settings = self._wl._load_user_settings()
             settings._apply_user(user_settings)
 
+        # ensure that user settings don't set saving to true
+        # if user explicitly set these to false
+        if save_code_pre_user_settings is False:
+            settings.update({"save_code": False})
+
         # TODO(jhr): should this be moved? probably.
         d = dict(_start_time=time.time(), _start_datetime=datetime.datetime.now(),)
         settings.update(d)
 
+        self._log_setup(settings)
+
         if settings._jupyter:
             self._jupyter_setup(settings)
 
-        self._log_setup(settings)
-
         self.settings = settings.freeze()
 
     def teardown(self):
@@ -238,6 +245,10 @@ class _WandbInit(object):
     def _pause_backend(self):
         if self.backend is not None:
             logger.info("pausing backend")
+            # Attempt to save the code on every execution
+            if self.notebook.save_ipynb():
+                res = self.run.log_code(root=None)
+                logger.info("saved code: %s", res)
             self.backend.interface.publish_pause()
 
     def _resume_backend(self):
@@ -247,9 +258,12 @@ class _WandbInit(object):
 
     def _jupyter_teardown(self):
         """Teardown hooks and display saving, called with wandb.finish"""
-        logger.info("cleaning up jupyter logic")
         ipython = self.notebook.shell
         self.notebook.save_history()
+        if self.notebook.save_ipynb():
+            self.run.log_code(root=None)
+            logger.info("saved code and history")
+        logger.info("cleaning up jupyter logic")
         # because of how we bind our methods we manually find them to unregister
         for hook in ipython.events.callbacks["pre_run_cell"]:
             if "_resume_backend" in hook.__name__:
@@ -291,6 +305,7 @@ class _WandbInit(object):
         filesystem._safe_makedirs(os.path.dirname(settings.log_internal))
         filesystem._safe_makedirs(os.path.dirname(settings.sync_file))
         filesystem._safe_makedirs(settings.files_dir)
+        filesystem._safe_makedirs(settings._tmp_code_dir)
 
         if settings.symlink:
             self._safe_symlink(
